use bevy::prelude::*;
use std::path::PathBuf;

use self::registry::CreateRegistryPlugin;

mod registry;

#[derive(Debug, Resource)]
pub struct BevyBlenderConfig {
    pub create_registry: bool,
    pub registry_path: PathBuf,
    pub type_filter: SceneFilter,
}

impl From<&BevyBlenderPlugin> for BevyBlenderConfig {
    fn from(value: &BevyBlenderPlugin) -> Self {
        Self {
            create_registry: value.create_registry,
            registry_path: value.registry_path.clone(),
            type_filter: value.type_filter.clone(),
        }
    }
}

pub struct BevyBlenderPlugin {
    create_registry: bool,
    registry_path: PathBuf,
    type_filter: SceneFilter,
}

impl Default for BevyBlenderPlugin {
    fn default() -> Self {
        Self {
            create_registry: true,
            registry_path: "registry.json".into(),
            type_filter: SceneFilter::default(),
        }
    }
}

impl Plugin for BevyBlenderPlugin {
    fn build(&self, app: &mut bevy::prelude::App) {
        #[cfg(debug_assertions)]
        if self.create_registry {
            app.add_plugins(CreateRegistryPlugin);
        }

        app.insert_resource(BevyBlenderConfig::from(self));
    }
}
